==> create-fly-1.21.10-6.0.8-4-sources.jar: com/zurrtum/create/content/contraptions/actors/contraptionControls/ContraptionControlsMovingInteraction.java <==
package com.zurrtum.create.content.contraptions.actors.contraptionControls;

import com.zurrtum.create.AllClientHandle;
import com.zurrtum.create.AllSoundEvents;
import com.zurrtum.create.api.behaviour.interaction.MovingInteractionBehaviour;
import com.zurrtum.create.api.behaviour.movement.MovementBehaviour;
import com.zurrtum.create.content.contraptions.AbstractContraptionEntity;
import com.zurrtum.create.content.contraptions.Contraption;
import com.zurrtum.create.content.contraptions.actors.contraptionControls.ContraptionControlsMovement.ElevatorFloorSelection;
import com.zurrtum.create.content.contraptions.behaviour.MovementContext;
import com.zurrtum.create.content.contraptions.elevator.ElevatorContraption;
import com.zurrtum.create.content.trains.entity.Carriage;
import com.zurrtum.create.content.trains.entity.CarriageContraptionEntity;
import com.zurrtum.create.content.trains.entity.Train;
import com.zurrtum.create.infrastructure.packet.c2s.ElevatorTargetFloorPacket;
import com.zurrtum.create.infrastructure.packet.s2c.ContraptionDisableActorPacket;
import org.apache.commons.lang3.tuple.MutablePair;

import java.util.Iterator;
import java.util.List;
import net.minecraft.class_1268;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_243;
import net.minecraft.class_3218;
import net.minecraft.class_3489;
import net.minecraft.class_3499.class_3501;

public class ContraptionControlsMovingInteraction extends MovingInteractionBehaviour {
    @Override
    public boolean handlePlayerInteraction(class_1657 player, class_1268 activeHand, class_2338 localPos, AbstractContraptionEntity contraptionEntity) {
        Contraption contraption = contraptionEntity.getContraption();

        MutablePair<class_3501, MovementContext> actor = contraption.getActorAt(localPos);
        if (actor == null)
            return false;
        MovementContext ctx = actor.right;
        if (ctx == null)
            return false;
        if (contraption instanceof ElevatorContraption ec)
            return elevatorInteraction(player, localPos, contraptionEntity, ec, ctx);
        if (contraptionEntity.method_73183().method_8608()) {
            if (AllClientHandle.INSTANCE.getBlockEntityClientSide(contraption, ctx.localPos) instanceof ContraptionControlsBlockEntity cbe)
                cbe.pressButton();
            return true;
        }

        class_1799 filter = ContraptionControlsMovement.getFilter(ctx);
        boolean disable = true;
        boolean invert = false;

        List<class_1799> disabledActors = contraption.getDisabledActors();
        for (Iterator<class_1799> iterator = disabledActors.iterator(); iterator.hasNext(); ) {
            class_1799 presentFilter = iterator.next();
            boolean sameFilter = ContraptionControlsMovement.isSameFilter(presentFilter, filter);
            if (presentFilter.method_7960()) {
                iterator.remove();
                disable = false;
                if (!sameFilter)
                    invert = true;
                continue;
            }
            if (!sameFilter)
                continue;
            iterator.remove();
            disable = false;
            break;
        }

        if (invert) {
            for (MutablePair<class_3501, MovementContext> pair : contraption.getActors()) {
                MovementBehaviour behaviour = MovementBehaviour.REGISTRY.get(pair.left.comp_1342());
                if (behaviour == null)
                    continue;
                class_1799 behaviourStack = behaviour.canBeDisabledVia(pair.right);
                if (behaviourStack == null)
                    continue;
                if (ContraptionControlsMovement.isSameFilter(behaviourStack, filter))
                    continue;
                if (contraption.isActorTypeDisabled(behaviourStack))
                    continue;
                disabledActors.add(behaviourStack);
                send(contraptionEntity, behaviourStack, true);
            }
        }

        if (filter.method_7960())
            disabledActors.clear();
        if (disable)
            disabledActors.add(filter);

        contraption.setActorsActive(filter, !disable);
        ContraptionControlsBlockEntity.sendStatus(player, filter, !disable);
        send(contraptionEntity, filter, disable);

        AllSoundEvents.CONTROLLER_CLICK.play(
            player.method_73183(),
            null,
            class_2338.method_49638(contraptionEntity.toGlobalVector(class_243.method_24953(localPos), 1)),
            1,
            disable ? 0.8f : 1.5f
        );

        if (!(contraptionEntity instanceof CarriageContraptionEntity cce))
            return true;
        if (!filter.method_31573(class_3489.field_15553))
            return true;

        // Special case: Doors are toggled on all carriages of a train
        Carriage carriage = cce.getCarriage();
        Train train = carriage.train;
        for (Carriage c : train.carriages) {
            CarriageContraptionEntity anyAvailableEntity = c.anyAvailableEntity();
            if (anyAvailableEntity == null)
                continue;
            Contraption cpt = anyAvailableEntity.getContraption();
            cpt.setActorsActive(filter, !disable);
            ContraptionControlsBlockEntity.sendStatus(player, filter, !disable);
            send(anyAvailableEntity, filter, disable);
        }

        return true;
    }

    private void send(AbstractContraptionEntity contraptionEntity, class_1799 filter, boolean disable) {
        class_3218 world = (class_3218) contraptionEntity.method_73183();
        world.method_14178()
            .method_18754(contraptionEntity, new ContraptionDisableActorPacket(contraptionEntity.method_5628(), filter, !disable));
    }

    private boolean elevatorInteraction(
        class_1657 player,
        class_2338 localPos,
        AbstractContraptionEntity contraptionEntity,
        ElevatorContraption contraption,
        MovementContext ctx
    ) {
        class_1937 level = contraptionEntity.method_73183();
        if (!level.method_8608()) {
            class_2338 pos = class_2338.method_49638(contraptionEntity.toGlobalVector(class_243.method_24953(localPos), 1));
            AllSoundEvents.CONTROLLER_CLICK.play(level, null, pos, 1, 1.5f);
            AllSoundEvents.CONTRAPTION_ASSEMBLE.play(level, null, pos, 0.75f, 0.8f);
            return true;
        }
        if (!(ctx.temporaryData instanceof ElevatorFloorSelection efs))
            return false;
        if (efs.currentTargetY == contraption.clientYTarget)
            return true;

        AllClientHandle.INSTANCE.sendPacket(player, new ElevatorTargetFloorPacket(contraptionEntity, efs.currentTargetY));
        if (AllClientHandle.INSTANCE.getBlockEntityClientSide(contraption, ctx.localPos) instanceof ContraptionControlsBlockEntity cbe)
            cbe.pressButton();
        return true;
    }

}

==> create-fly-1.21.11-6.0.8-2-sources.jar: com/zurrtum/create/content/contraptions/actors/contraptionControls/ContraptionControlsMovingInteraction.java <==
package com.zurrtum.create.content.contraptions.actors.contraptionControls;

import com.zurrtum.create.AllClientHandle;
import com.zurrtum.create.AllSoundEvents;
import com.zurrtum.create.api.behaviour.interaction.MovingInteractionBehaviour;
import com.zurrtum.create.api.behaviour.movement.MovementBehaviour;
import com.zurrtum.create.content.contraptions.AbstractContraptionEntity;
import com.zurrtum.create.content.contraptions.Contraption;
import com.zurrtum.create.content.contraptions.actors.contraptionControls.ContraptionControlsMovement.ElevatorFloorSelection;
import com.zurrtum.create.content.contraptions.behaviour.MovementContext;
import com.zurrtum.create.content.contraptions.elevator.ElevatorContraption;
import com.zurrtum.create.content.trains.entity.Carriage;
import com.zurrtum.create.content.trains.entity.CarriageContraptionEntity;
import com.zurrtum.create.content.trains.entity.Train;
import com.zurrtum.create.infrastructure.packet.c2s.ElevatorTargetFloorPacket;
import com.zurrtum.create.infrastructure.packet.s2c.ContraptionDisableActorPacket;
import org.apache.commons.lang3.tuple.MutablePair;

import java.util.Iterator;
import java.util.List;
import net.minecraft.class_1268;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_243;
import net.minecraft.class_3218;
import net.minecraft.class_3489;
import net.minecraft.class_3499.class_3501;

public class ContraptionControlsMovingInteraction extends MovingInteractionBehaviour {
    @Override
    public boolean handlePlayerInteraction(
        class_1657 player,
        class_1268 activeHand,
        class_2338 localPos,
        AbstractContraptionEntity contraptionEntity
    ) {
        Contraption contraption = contraptionEntity.getContraption();

        MutablePair<class_3501, MovementContext> actor = contraption.getActorAt(localPos);
        if (actor == null)
            return false;
        MovementContext ctx = actor.right;
        if (ctx == null)
            return false;
        if (contraption instanceof ElevatorContraption ec)
            return elevatorInteraction(player, localPos, contraptionEntity, ec, ctx);
        if (contraptionEntity.method_73183().method_8608()) {
            if (AllClientHandle.INSTANCE.getBlockEntityClientSide(contraption, ctx.localPos) instanceof ContraptionControlsBlockEntity cbe)
                cbe.pressButton();
            return true;
        }

        class_1799 filter = ContraptionControlsMovement.getFilter(ctx);
        boolean disable = true;
        boolean invert = false;

        List<class_1799> disabledActors = contraption.getDisabledActors();
        for (Iterator<class_1799> iterator = disabledActors.iterator(); iterator.hasNext(); ) {
            class_1799 presentFilter = iterator.next();
            boolean sameFilter = ContraptionControlsMovement.isSameFilter(presentFilter, filter);
            if (presentFilter.method_7960()) {
                iterator.remove();
                disable = false;
                if (!sameFilter)
                    invert = true;
                continue;
            }
            if (!sameFilter)
                continue;
            iterator.remove();
            disable = false;
            break;
        }

        if (invert) {
            for (MutablePair<class_3501, MovementContext> pair : contraption.getActors()) {
                MovementBehaviour behaviour = MovementBehaviour.REGISTRY.get(pair.left.comp_1342());
                if (behaviour == null)
                    continue;
                class_1799 behaviourStack = behaviour.canBeDisabledVia(pair.right);
                if (behaviourStack == null)
                    continue;
                if (ContraptionControlsMovement.isSameFilter(behaviourStack, filter))
                    continue;
                if (contraption.isActorTypeDisabled(behaviourStack))
                    continue;
                disabledActors.add(behaviourStack);
                send(contraptionEntity, behaviourStack, true);
            }
        }

        if (filter.method_7960())
            disabledActors.clear();
        if (disable)
            disabledActors.add(filter);

        contraption.setActorsActive(filter, !disable);
        ContraptionControlsBlockEntity.sendStatus(player, filter, !disable);
        send(contraptionEntity, filter, disable);

        AllSoundEvents.CONTROLLER_CLICK.play(
            player.method_73183(),
            null,
            class_2338.method_49638(contraptionEntity.toGlobalVector(class_243.method_24953(localPos), 1)),
            1,
            disable ? 0.8f : 1.5f
        );

        if (!(contraptionEntity instanceof CarriageContraptionEntity cce))
            return true;
        if (!filter.method_31573(class_3489.field_15553))
            return true;

        // Special case: Doors are toggled on all carriages of a train
        Carriage carriage = cce.getCarriage();
        Train train = carriage.train;
        for (Carriage c : train.carriages) {
            CarriageContraptionEntity anyAvailableEntity = c.anyAvailableEntity();
            if (anyAvailableEntity == null)
                continue;
            Contraption cpt = anyAvailableEntity.getContraption();
            cpt.setActorsActive(filter, !disable);
            ContraptionControlsBlockEntity.sendStatus(player, filter, !disable);
            send(anyAvailableEntity, filter, disable);
        }

        return true;
    }

    private void send(AbstractContraptionEntity contraptionEntity, class_1799 filter, boolean disable) {
        class_3218 world = (class_3218) contraptionEntity.method_73183();
        world.method_14178()
            .method_18754(contraptionEntity, new ContraptionDisableActorPacket(contraptionEntity.method_5628(), filter, !disable));
    }

    private boolean elevatorInteraction(
        class_1657 player,
        class_2338 localPos,
        AbstractContraptionEntity contraptionEntity,
        ElevatorContraption contraption,
        MovementContext ctx
    ) {
        class_1937 level = contraptionEntity.method_73183();
        if (!level.method_8608()) {
            class_2338 pos = class_2338.method_49638(contraptionEntity.toGlobalVector(class_243.method_24953(localPos), 1));
            AllSoundEvents.CONTROLLER_CLICK.play(level, null, pos, 1, 1.5f);
            AllSoundEvents.CONTRAPTION_ASSEMBLE.play(level, null, pos, 0.75f, 0.8f);
            return true;
        }
        if (!(ctx.temporaryData instanceof ElevatorFloorSelection efs))
            return false;
        if (efs.currentTargetY == contraption.clientYTarget)
            return true;

        AllClientHandle.INSTANCE.sendPacket(player, new ElevatorTargetFloorPacket(contraptionEntity, efs.currentTargetY));
        if (AllClientHandle.INSTANCE.getBlockEntityClientSide(contraption, ctx.localPos) instanceof ContraptionControlsBlockEntity cbe)
            cbe.pressButton();
        return true;
    }

}
